"""Defines the Region for Diablo III

Classes:
    Region

Misc Variables:
    __version__
    __author__

Author: David "Gahd" Couples
License: GPL v3
Copyright: February 24, 2022
"""
from battlenet_client.constants import Region as BaseRegion


__version__ = '2.0.0'
__author__ = 'David \'Gahd\' Couples'


class Region(BaseRegion):
    """Defines the constants for regions"""
    pass
